function CERW_batch_postprocess(input_dir,output_dir,post_opt)

fprintf('postprocessing images...\n');

if ischar(post_opt) || isstring(post_opt)
    in_opt = load(post_opt);
    if isfield(in_opt,'post_opt')
        post_opt = in_opt.post_opt;
    elseif isfield(in_opt,'options')
        post_opt = in_opt.options;
    end
end

fprintf('formatting options...\n');

post_opt = generate_options(post_opt,input_dir,output_dir,'postprocess');

input_files = dir(input_dir);
input_files(ismember({input_files.name}, {'.','..'})) = [];

n_files = length(input_files);

fprintf('processing %d frames.\n', n_files);

for i = 1:length(input_files)
    if input_files(i).isdir
        postprocess_subdir(input_dir,input_files(i).name,output_dir,post_opt);
    end
end
end

%%
function postprocess_subdir(input_parent,input_subdir,output_dir,options)

fprintf('processing frame %s\n', input_subdir);
tic

input_dir = fullfile(input_parent,input_subdir);
in_extn = '.pgm';
b_regex = '(\d+)';
k_regex = '(\d\.\d+)';
out_extn = '.tif';
naming = 'standard';
raw_s = 0;
raw_dir = '';

if isfield(options,'in_extn')
    in_extn = char(options.in_extn);
end
if isfield(options,'naming')
    naming = options.naming;
end
if isfield(options,'raw_s')
    raw_s = options.raw_s;
end
if isfield(options, 'raw_dir')
    raw_dir = options.raw_dir;
end

% restrict post processing to specified values of beta and kappa
if isfield(options,'beta')
    if ischar(options.beta) || isstring(options.beta)
        b_regex = strcat('(',options.beta,')');
    end
end
if isfield(options,'kappa')
    if ischar(options.kappa) || isstring(options.kappa)
        k_regex = strcat('(',options.kappa,')');
    end
end
% only supported extensions are tif and pgm
if (~isequal(in_extn,'.tif') && ~isequal(in_extn,'.pgm'))
    error('Input extension must be tif or pgm');
end

[input_names,image_tokens] = get_files(input_dir,strcat('(\w+)',b_regex,'(\w+)',k_regex,'\',in_extn));
file_count = numel(input_names);

if(file_count > 0)
    if exist(output_dir,'dir') == 0
        mkdir(output_dir);
    end
    image_names = image_tokens(:,1);
    if strcmp(naming,'CTC')
        image_numbers = string(regexp(image_names,'(\d+)','match'));
        output_names = strcat('mask',image_numbers,out_extn);
    elseif strcmp(naming,'standard')
        for i = 2:size(image_tokens,2)
            image_names = strcat(image_names,image_tokens(:,i));
        end
        safe_names = strrep(image_names,'.','_');
        output_names = strcat(safe_names,out_extn);
    end
else
    warning('No segmentation files found for directory %s',input_dir);
end

for i = 1:file_count
    input_path = fullfile(input_dir, input_names(i));
    if strcmp(in_extn,'.tif')
        im_in = imread3(input_path);
    elseif strcmp(in_extn,'.pgm')
        im_in = pgmread3(input_path);
    end
    if exist(raw_dir,'dir')~=0
        raw_path = fullfile(raw_dir,strcat(input_subdir,'.tif'));
        im_raw = imread3(raw_path);
        im_raw = imresize3(im_raw,size(im_in));
        if raw_s > 0
            im_raw = imgaussfilt3(im_raw,raw_s);
        end
        im_grad = imgradient3(im_raw);
        im_out = CERW_postprocess_image(im_in,options, im_grad) * 1;
    else
        im_out = CERW_postprocess_image(im_in,options) * 1;
    end
    if strcmp(naming,'CTC')
        im_out = uint16(im_out);
    end
    output_path = char(fullfile(output_dir,output_names(i)));
    imwrite(im_out(:,:,1),output_path);
    for k = 2:size(im_out,3)
        imwrite(im_out(:,:,k),output_path,'WriteMode','append');
    end
end
toc
end
