function CERW_batch_setup(input_path,pre_dir,seed_dir,pre_opt,seed_opt)

if ischar(pre_opt) || isstring(pre_opt)
    in_opt = load(pre_opt);
    if isfield(in_opt,'options')
        pre_opt = in_opt.options;
    end
    if isfield(in_opt,'pre_opt')
        pre_opt = in_opt.pre_opt;
    end
    if isfield(in_opt,'seed_opt')
        seed_opt = in_opt.seed_opt;
    end
end
if exist('seed_opt','var') && (ischar(seed_opt) || isstring(seed_opt))
    in_opt = load(seed_opt);
    if isfield(in_opt,'seed_opt')
        seed_opt = in_opt.seed_opt;
    elseif isfield(in_opt,'options')
        seed_opt = in_opt.options;
    end
end

fprintf('formatting options...\n');
% create new preprocessing options to fill in options not in pre_opt and
% add a description.
pre_opt = generate_options(pre_opt,input_path,pre_dir,'preprocess');
% create new seeds options to fill in options not in seed_opt and add a
% description
seed_opt = generate_options(seed_opt,input_path,seed_dir,'seed');

file_names = union(pre_opt.file_names,seed_opt.file_names);
image_numbers = union(pre_opt.image_numbers,seed_opt.image_numbers);
frame_count = numel(image_numbers);

if strcmp(pre_opt.input_mode,'file')
    [~,img_name,~] = fileparts(input_path);
end

fprintf('processing %d frames\n',frame_count);

if frame_count > 0
    if exist(pre_dir,'dir')==0
        mkdir(pre_dir);
    end
    if exist(seed_dir,'dir')==0
        mkdir(seed_dir);
    end
end

for i = 1:frame_count
    tic
    if strcmp(pre_opt.input_mode,'dir')
        file_path = fullfile(input_path,file_names(i));
        [~,image_name,~] = fileparts(file_path);
        im_in = imread3(file_path);
    else
        error("Input mode option %s not recognized",pre_opt.input_mode);
    end
    pre_name = strcat(image_name,pre_opt.out_extn);
    pre_path = fullfile(pre_dir,pre_name);
    im_pre = CERW_preprocess_image(im_in,pre_opt);
    pgmwrite3(im_pre,pre_path);
    
    seed_name = strcat(image_name,seed_opt.out_extn);
    seed_path = fullfile(seed_dir,seed_name);
    im_seed = CERW_seeds(im_in,seed_opt);
    pgmwrite3(im_seed,seed_path);
    fprintf('Image %s processed\n',image_name);
    toc
end

end

