function im_out = CERW_postprocess_image(im_in,options,im_grad)

T_image = 0.5;  % threshold for image normalized to [0,1]
z_scale = 1;    % scaling from segmented to real
r_erode = 0;    % radius of erosion of segmented image
r_dilate = 0;   % radius of dilation of segmented image
do_fill = true; % fill holes in segmented image
do_cc = true;   % use largest connected component of image
r_smart = 0;    % use smart surface adjustment


if isfield(options, 'threshold');   T_image = options.threshold;    end
if isfield(options, 'z_scale');     z_scale = options.z_scale;      end
if isfield(options, 'r_erode');     r_erode = options.r_erode;      end
if isfield(options, 'r_dilate');    r_dilate = options.r_dilate;    end
if isfield(options, 'do_fill');     do_fill = options.do_fill;      end
if isfield(options, 'do_cc') ;      do_cc = options.do_cc;          end
if nargin > 2 && isfield(options, 'r_smart')
    r_smart = options.r_smart;
end

sz = size(im_in);
im_in = im_in - min(im_in(:));
im_in = im_in / max(im_in(:));

im_out = (im_in > T_image);

if(r_dilate > 0)
    im_dist = bwdist(im_out>0);
    im_out = (im_dist < r_dilate);
end

if(do_fill)
    im_out = imfill(im_out,'holes');
end

if(r_erode > 0)
    im_dist = bwdist(im_out==0);
    im_out = (im_dist > r_erode);
end

if(r_smart > 0)
    im_out = medfilt3(surface_intensity_adjust(im_out,im_grad,r_smart));
end

if(z_scale ~= 1)
    sz = sz .* [1,1,z_scale];
    im_out = imresize3(im_out * 1,sz,'Method','nearest');
end

if(do_cc)
    cc = bwconncomp(im_out,26);
    num_pixels = cellfun(@numel,cc.PixelIdxList);
    [~,max_idx] = max(num_pixels);
    for i = 1:numel(num_pixels)
        if i ~= max_idx
            im_out(cc.PixelIdxList{i}) = 0;
        end
    end
end


end