function options = generate_options(in_opt,in_path,out_dir,type)

options = generate_default_options(type);

% setup io options
options.input_path = in_path;
options.output_dir = out_dir;


if isfolder(in_path)
    if strcmp(type,'postprocess')
        [file_names,image_names] = get_files(in_path,'(\w+)');
    else
        [file_names,image_names] = get_files(in_path,strcat('(\w+)',options.in_extn));
    end
    image_numbers = get_frame_numbers(image_names);
    n_count = size(image_numbers,2);
    match_count = zeros(n_count,1);
    for i = 1:n_count
        if isfield(in_opt,'image_numbers')
            match_count(i) = sum(ismember(in_opt.image_numbers,image_numbers(:,i)));
        else
            match_count(i) = numel(unique(image_numbers(:,i)));
        end
    end
    [~,max_idx] = max(match_count);
    image_numbers = image_numbers(:,max_idx);
    if isfield(in_opt,'image_numbers')
        keep_numbers = ismember(image_numbers,in_opt.image_numbers);
        image_numbers = image_numbers(keep_numbers);
        file_names = file_names(keep_numbers);
    end
    options.file_names = file_names;
    options.image_numbers = image_numbers;
    options.input_mode = 'dir';
elseif isfile(in_path)
    [~,image_name,extn] = fileparts(in_path);
    options.file_names = strcat(image_name,extn);
    if isfield(in_opt,'image_numbers')
        options.image_numbers = in_opt.image_numbers;
    else
        im_info = imfinfo(in_path);
        options.frame_count = double(get_tif_parameter(im_info(1),'frames'));
        options.image_numbers = 1:options.frame_count;
    end
    options.input_mode = 'file';
end


options.frame_count = numel(options.image_numbers);

in_names = string(fieldnames(in_opt));

for i = 1:numel(in_names)
    if ~isfield(options,in_names(i))
        warning("Option %s not found for options struct of type %s",in_names(i),type);
    end
    options = setfield(options,in_names(i),getfield(in_opt,in_names(i)));
end

end

function options =  generate_default_options(type)

options.description = '';
options.input_path = '';
options.output_dir = '';
options.input_mode = 'dir';
options.in_extn = '.tif';
options.out_extn = '.pgm';
options.image_names = [];
options.image_numbers = [];
options.frame_count = 0;

if nargin>0 && (isstring(type) || ischar(type))
    if strcmp(type,'preprocess')
        options.z_scale = 1;

        options.gamma = 1.0;
        options.bp_S = 0;
        options.bp_B = 0;
        options.clahe_width = 1;
        options.s_gauss = 0;
        options.r_mean = 0;
        options.r_median = 0;

        options.color_mapping = 'rgb';
        options.input_channels = 1;
        options.output_channel = 1;
        options.channel_weights = [1,1,1];
    elseif strcmp(type,'seed')
        options = generate_default_options('preprocess');
        options.phan_radius = 10;
        options.phan_r = 1;
        options.phan_p = 2;
        options.phan_restrict = false;
        sub_options = options;
        options.FG_options = sub_options;
        options.BG_options = sub_options;
    elseif strcmp(type,'postprocess')
        options.in_extn = '.pgm';
        options.out_extn = '.tif';
        options.naming = 'standard';
        options.beta = [];
        options.kappa = [];
        options.threshold = 0.5;
        options.z_scale = 1;
        options.r_erode = 0;
        options.r_dilate = 0;
        options.do_fill = true;
        options.do_cc = true;
        options.raw_s = 0;
        options.raw_dir = '';
        options.r_smart = 0;
    else
        warning("options type %s not found.", type);
    end
    options.description = 'type';
end
end