function A = imread3(im_path,x_min,y_min,z_min,width,height,depth)
%imread3 - read a 3D tif file and crop
%   
%   'A'         - output 3D image
%   'im_path'  - path to file
%   'x_min'     - x start position of cropping rectangle
%   'width'     - width of cropping rectangle
%   'y_min'     - y start position of cropping rectangle
%   'height'    - height of cropping rectangle
%   'z_min'     - z start position of cropping rectangle
%   'depth'    - depth of cropping rectangle



im_info = imfinfo(char(im_path));
im_depth = numel(im_info);
im_width = im_info.Width;
im_height = im_info.Height;

if exist('x_min','var') && numel(x_min) == 1 && x_min > 0
    if ~exist('width','var') || numel(width) ~= 1 || x_min + width > im_width
        width = im_width - x_min + 1;
    end
else
    x_min = 1;
    width = im_width;
end

if exist('y_min','var') && numel(y_min) == 1 && y_min > 0
    if ~exist('height','var') || numel(height) ~= 1 || y_min + height > im_height
        height = im_height - y_min + 1;
    end
else
    y_min = 1;
    height = im_height;
end

if exist('z_min','var') && numel(z_min) == 1 && z_min > 0
    if ~exist('depth','var') || numel(depth) ~= 1 || z_min + depth > im_depth
        depth = im_depth - z_min + 1;
    end
else
    z_min = 1;
    depth = im_depth;
end

channels = 1;
if isequal(im_info.ColorType, 'truecolor')
    channels = 3;
end

A = zeros(height, width, depth,channels);
for z = z_min:(z_min + depth - 1)
    if channels == 1
        A(:,:,z - z_min + 1) = imcrop(imread(im_path, z),[x_min,y_min,width - 1,height - 1]);
    else
        A(:,:,z - z_min + 1,:) = permute(imcrop(imread(im_path, z),[x_min,y_min,width - 1,height - 1]),[1,2,4,3]);
    end
end
    
end

