function pgmwrite3(A,path)
% writes a 3D pgm file for use in power watershed software
%
%   'A'     -   3D image to be written
%   'path'  -   path of image file (including extension)
%
% 3D pgm files have the following header:
% 'P5
% width height depth
% max_value'
% where
% P5 is a pgm formatting parameter
% width,height, and depth are the dimensions of the 3D image
% max_value is the maximum value of the image.
% images are written as 'uint8' values, and images are normalized such that
% max_value = 255
% and the minimum value is zero

% get dimensions
[height,width,depth] = size(A);

% convert to double for normalizing
A = double(A);
%normalize to [0,255] and convert to uint8
min_A = min(A(:));
A = A - min_A;
max_A = max(A(:));
A = uint8(255 * A / max_A);

%change x and y axes for writing
A = permute(A,[2,1,3]);

% create header char array
line_1 = 'P5';
line_2 = num2str([width,height,depth]);
line_3 = '255';
header_lines = [line_1,newline,line_2,newline,line_3,newline];

% write the header and data
fid = fopen(path,'w');
fprintf(fid,header_lines);
fwrite(fid,A(:));
fclose(fid);

end