function im_out = surface_intensity_adjust(im_bin,im_input,cortex_width)

[f,v] = isosurface(im_bin);
sz_im = size(im_bin);

cortex_line = -cortex_width:cortex_width; 
TR = triangulation(f,v);
v_norm = vertexNormal(TR);
l_x = round(v(:,1) + v_norm(:,1).*cortex_line);
l_y = round(v(:,2) + v_norm(:,2).*cortex_line);
l_z = round(v(:,3) + v_norm(:,3).*cortex_line);
l_x(l_x > sz_im(2)) = sz_im(2);
l_x(l_x < 1) = 1;
l_y(l_y > sz_im(1)) = sz_im(1);
l_y(l_y < 1) = 1;
l_z(l_z > sz_im(3)) = sz_im(3);
l_z(l_z < 1) = 1;
line_idx = sub2ind(sz_im,l_y,l_x,l_z);

% only consider normal as distance to surface is increasing
l_stop = false(size(line_idx));
im_dist = bwdist(im_bin>0) + bwdist(im_bin==0);
for i = 1:(cortex_width-1)
    i0 = cortex_width + 1 + i;
    i1 = i0 + 1;
    l_stop(im_dist(line_idx(:,i1))<im_dist(line_idx(:,i0)),i1:end) = true;
    i0 = cortex_width + 1 - i;
    i1 = i0 - 1;
    l_stop(im_dist(line_idx(:,i1))<im_dist(line_idx(:,i0)),1:i1) = true;
end
line_idx(l_stop) = 1;

% dummy voxel, assumed to be not of interest
im_input(1) = 0;
    line_scan = im_input(line_idx); 
[~,max_idx] = max(line_scan,[],2);

max_med = zeros(size(max_idx));
v_count = size(v,1);
e = edges(TR);    
v_neigh{v_count} = [];
for i = 1:v_count
    v_neigh{i} = [i;e(e(:,1)==i,2);e(e(:,2)==i,1)];
end
n_neigh = cellfun(@numel,v_neigh);

for i = unique(n_neigh)
    v_idx = find(n_neigh==i);
    nbrs = cell2mat(v_neigh(v_idx));
    max_med(v_idx) = median(max_idx(nbrs))';
end
max_idx = max_med;

im_out = im_bin;
for i = 1:size(cortex_line,2)
    im_out(line_idx(:,i)) = (max_idx <= i);
end

end